#' @title Perfom hydraulic analysis for a model
#' 
#' @description
#' `analyse_hydraulic()` perfoms hydraulic analysis for a model.
#'
#' @param model either a character, name given to the model or an object of
#' 		class model
#' @param limnigraph numeric, matrix of floodwater depth heights by instant t
#' 		for each exterior opening exposed to flooding
#' @param flood_duration numeric, duration of the flood event in seconds,
#' 		optional; if not provided, calculated using limnigraph
#' @param opening_scenario character, one of the following options: "open",
#' 		"closed", "combined". See details.
#' @param dt_max numeric, optional; maxmimum timestep for simulation
#' @param Cd numeric, optional; discharge coefficient for each opening,
#' 		default to getOption("floodam_building_hydraulic_discharge_coefficient")
#' @param sim_id character, optional; id of simulation if function called from a
#' 		loop
#' @param stage character, what are the stages that should be done, default to nothing
#' @param what character, outputs to be saved
#' @param light boolean, if true only return hmax and damage
#' @param verbose boolean, will floodam tells what it is doing, default to
#' 		getOption("floodam_building_verbose")
#'
#' @details
#'
#' `opening_scenario` controls the state of openings:
#' 
#' - "open": scenario where all openings are open
#' - "closed": scenario where all openings are closed
#' - "combined": scenario where all exterior openings are closed and all interior
#' openings are open
#'
#' If `flood_duration` is missing, it is computed from `limnigraph` (max time +
#'  half an hour).
#' 
#' @return
#' 
#' list of matrix:
#' 
#' - `h`: water depths in each rooms for each time step
#' - `eQ`: water exchanges through openings for each time step
#' - `v`: velocity of exchanges through openings for each time step
#' - `eS`: wet surface of exchanges through openings for each time step
#' - `hmax` : peak water depths in each rooms and their peak time
#' - `dangerosity` (optional) : dangerosity when opening a door
#' - `damage` (optional) : damage per room based on peak water depth
#' 
#' @examples
#' 
#' # declaring input and output paths
#' model_path = list(
#' 	data = system.file("extdata", package = "floodam.building"),
#' 	output = tempdir()
#' )
#' 
#' # analyzing model 'adu_t' of type 'adu' using the sequential steps *load* 
#' # (load model data), *extract* (extract model data) and *hydraulic* 
#' model = analyse_model(
#' 	model = "adu_t", 
#' 	type = "adu", 
#' 	stage = c("load", "extract", "hydraulic"), 
#' 	path = model_path
#' )
#' 
#' # generate limnigraph
#' flood = generate_limnigraph(
#'   time = c(0, 300, 900),
#'   depth = cbind(external_1 = c(0, 3, 0)),
#'   external = list(
#'     external_1 = c("wall_A", "wall_B", "wall_C", "wall_D", "wall_E", "wall_F", "wall_G", "wall_H"))
#' )
#' 
#' hydraulic = analyse_hydraulic(
#' 	model = model,
#' 	limnigraph = flood,
#' 	sim_id = "test",
#' 	stage = "hydraulic"
#' )
#'
#' @export

analyse_hydraulic = function(
		model,
		limnigraph,
		flood_duration,
		opening_scenario = c("combined", "close", "open"),
		dt_max = 0.5,
		Cd = 0.42,
		sim_id = NULL,
		stage = "",
		what = c("hmax", "damage"),
		light = FALSE,
		verbose = getOption("floodam_building_verbose")
	)
{
	start = Sys.time()
	if (identical(stage, "all")) stage = getOption("floodam_building_hydrau_stage")
	stage = intersect(as.character(unlist(stage)), getOption("floodam_building_hydrau_stage"))

    if (length(stage) == 0) {
        if (verbose) message("No admissible stage asked. Nothing is done.\n")
        return(NULL)
    }

	# Perform hydraulic analysis
	if ("hydraulic" %in% stage) {
		# initial checks
		opening_scenario = match.arg(opening_scenario)
		if (missing(flood_duration)) {
			flood_duration = 2 * max(limnigraph[["limnigraph"]][, "time"]) + 1800
		}

		if (verbose) {
			message(
				sprintf("Simulating hydraulics for '%s'...\n", model[["name"]]),
				appendLF = FALSE
			)
		}
		
		# determine opening scenraio
		data_hyd = prepare_hydraulic_input(
			model,
			external = limnigraph[["external"]]
		)
		opening = data_hyd[[opening_scenario]]

		# adding informations for the simulation
		opening[["discharge_coeff"]] = Cd
		opening[["H_top"]] = opening[["H_abs"]] + opening[["height"]]
		room = data_hyd[["room"]]
		room[["room"]] = as.character(room[["room"]])

		r_names_to_merge = NULL
		# in case of invisible walls, merge rooms
		if("empty" %in% model[["data_table"]][["wall"]][["type"]])
		{
			r_names_to_merge = find_rooms_to_merge(model)

			if (verbose) {
				message(
					sprintf(
						"\t... found invisible walls between rooms for '%s'",
						model[["name"]]
					)
				)
			}
		}

		# starting the simulation
		result = compute_hydraulic(
			opening = opening,
			room = room,
			bound = limnigraph[["limnigraph"]],
			dt_max = dt_max,
			t_start = 0,
			t_max = flood_duration,
			r_names_to_merge = r_names_to_merge,
			light = light
		)

		class(result) = "hydraulic"

		if (verbose) {
			message(
				sprintf(
					"\t... hydraulics successfully modeled for '%s'",
					model[["name"]]
				)
			)
		}
	}

	# compute dangerosity
	if ("dangerosity" %in% stage) {
		opening = data_hyd[["open"]]

		# adding informations for the simulation
		opening[["discharge_coeff"]] = Cd
		opening[["H_top"]] = opening[["H_abs"]] + opening[["height"]]
		result_dg = compute_danger(
			result_h = result[["h"]],
			opening = opening,
			room = room,
			bound = limnigraph[["limnigraph"]],
			dt_max = dt_max,
			t_max = flood_duration)
		result[["danger"]] = result_dg

		message(
			sprintf(
				"\t... dangerosity successfully computed for '%s'",
				model[["name"]]
			)
		)
	}

	# compute damage
	if ("damaging" %in% stage) {
		if (is.null(model[["damaging"]][["absolute_by_room"]])) {
			stop("you need to enable parameter `dam_by_room` in analyse_model()")
		}
		result_dam = get_hydraulic_damage(result[["hmax"]]["z", ], model, limnigraph[["external"]])
		result[["damage"]] = result_dam

		message(
			sprintf(
				"\t... damaging successfully computed for '%s'",
				model[["name"]]
			)
		)
	}

	# Saving outputs
	if ("save" %in% stage) {
		name = sprintf(
			"%s-%s-%s",
			model[['name']],
			opening_scenario,
			if (is.null(sim_id)) "NA" else sim_id
		)
		path = model[["path"]][["model_output_hydraulic"]]
		for (what in intersect(names(result), what)) {
			data.table::fwrite(
				as.data.frame(result[[what]]),
				file.path(path, sprintf("%s-%s.csv.gz", name, what))
			)
		}
		message(
			sprintf(
				"\t... hydraulics successfully saved for '%s' in '%s'",
				model[["name"]],
				path
			)
		)
	}

	# Plotting something in display
	if ("display" %in% stage) {
		if(light == TRUE) {
			message(
				sprintf(
					"\t... can't display graphs when paramater `light` is TRUE"
				)
			)
		} else {
			if (verbose) message(sprintf("Display height view of '%s'", sim_id))
			plot(result, view = "height", device = "display")
		}
	}

	duration = Sys.time() - start

	if (verbose) {
		message(
			sprintf(
				"End of analysis for '%s'. Total elapsed time %.2f %s\n",
				model[['name']],
				duration,
				attr(duration, "units")
			)
		)
	}

	invisible(result)
}
