#' Analyse the hydraulics of the model
#'
#' @param opening hydraulic part of the model, containing informations on the 
#' 		exchange
#' @param room hydraulic part of the model, containing informations on the 
#' 		rooms
#' @param bound numeric, matrix of floodwater depth heights by instant t
#' 		for each exterior opening exposed to flooding
#' @param dt_max numeric, optional; maxmimum timestep for simulation
#' @param t_start numeric, start time of the simulation
#' @param t_max numeric, duration of the flood event in seconds,
#' 		optional; if not provided, calculated using limnigraph
#' @param r_names_to_merge list of room vectors to be merged
#' @param light boolean, if true only return hmax
#'
#' @return lists of hydraulic values: water exchange through openings and
#' 	floodwater depth in rooms
#'
#' @export

compute_hydraulic = function(opening, room, bound, dt_max, t_start = 0, t_max,
	r_names_to_merge = NULL, light = FALSE) {

	base_room = room
	base_room_name = base_room[["room"]]

	# merging rooms if needed
	if (!is.null(r_names_to_merge)) {
		merged_data = merge_rooms(opening, room, r_names_to_merge)
		opening = merged_data[["opening"]]
		room = merged_data[["room"]]
	}

	# simplifying rooms with multiple levels
	merged_room = summed_room = room
	min_H_abs = stats::aggregate(H_abs ~ room, data = room, FUN = min)
	max_H_abs = stats::aggregate(H_abs ~ room, data = room, FUN = max)[["H_abs"]]
	room <- merge(room, min_H_abs, by = c("room", "H_abs"))

	# setting up names
	exterior_name = colnames(bound)[-1]
	room_name = room[["room"]]
	opening_name = opening[["opening"]]
	building_name = c(room_name, exterior_name)

	# creating surface function by elevation
	for (ro in row(merged_room)) {
		ro_name = merged_room[ro, "room"]
		summed_room[ro, ][["surface"]] = sum(merged_room[["surface"]][merged_room[["room"]] == ro_name & merged_room[["H_abs"]] <= merged_room[ro, ][["H_abs"]]])
	}
	summed_room = data.matrix(summed_room)
	rownames(summed_room) = merged_room[["room"]]
	summed_r_names = rownames(summed_room)

	# init building and boundaries
	building = rbind(
		surf = c(room[["surface"]],	rep(NA_real_, ncol(bound) - 1)),
		z0 = c(room[["H_abs"]], numeric(ncol(bound) - 1)),
		z = c(room[["initial_depth"]], bound[1, -1])
	)
	colnames(building) = building_name
	building["z", ] = building["z", ] + building["z0", ]

	if (any(room[["initial_depth"]] < 0)) stop("h cannot be negative")

	# init exchange
	exchange = rbind(
		width = opening[["width"]],
		z_bottom = opening[["H_abs"]],
		z_top = opening[["H_top"]], 
		Cd = opening[["discharge_coeff"]],
		z_up = 0,
		z_down = 0,
		flow = 0,
		s = 0
	)
	colnames(exchange) = opening_name

	# setting up the relations between rooms and exchange
	match_opening = t(as.matrix(opening[c("up", "down")]))
	colnames(match_opening) = opening_name
	matching = matrix(
		0, 
		nrow = nrow(opening),
		ncol = ncol(building),
		dimnames = list(opening_name, building_name)
	)
	for (o in opening_name) matching[o, match_opening[, o]] = c(-1, 1)

	# preallocating the output matrices
	nrows = floor(t_max / dt_max)
	result_h = matrix(
		NA_real_,
		nrow = nrows,
		ncol = ncol(building) + 1,
  		dimnames = list(c(), c("time", building_name)))
	result_q = matrix(
		NA_real_,
		nrow = nrows,
  		ncol = nrow(opening) + 1,
  		dimnames = list(c(), c("time", opening_name)))
	result_v = result_s = result_q
	result_h[1, 1] = result_q[1, 1] = result_s[1, 1] = result_v[1, 1] = t_start
	
	pos = 1

	# declaring the interpolation functions to compute water level on the outside
	approx_fun = sapply(
		exterior_name,
		function(x) {stats::approxfun(bound[, "time"], bound[, x], yright = 0)}
	)

	# time loop
	t_calc = t_start
	for (pos in seq(nrows)) {
		# get level for boundaries
		for (e in exterior_name) {
			z = approx_fun[[e]](t_calc)
			building["z", e] = z
		}
		# get level at exchange up and down
		exchange[c("z_up", "z_down"), ] = building["z", match_opening]
			
		# compute flow and section for all exchange
		exchange[c("s", "flow"), ] = rcpp_compute_exchange(exchange)
		# exchange[c("s", "flow"), ] = compute_exchange(exchange, opening_name)

		# compute dt_adj if needed
		dt_adj = dt_max
		Qtot = colSums(exchange["flow", ] * matching[, room_name])
		check = room_name[Qtot < 0]
		if (length(check) > 0) {
			volume_in = (building["z", check] - building["z0", check]) * building["surf", check]
			dt_adj = min(dt_adj, volume_in / -Qtot[check])
		}

		# get level for room
		building["z", room_name] = building["z", room_name] + Qtot * dt_adj / building["surf", room_name]

		# check room surfaces when there are merged rooms
		if (!is.null(r_names_to_merge)) {
			for (r in room_name) {
				row_data = summed_room[summed_r_names == r, ,drop = FALSE]
				surfs = row_data[, "surface"][building["z", r] >= row_data[, "H_abs"]]
				building["surf", r] = max(surfs)
			}
		}
	
		# updating the output matrices
		result_h[pos, ] = c(t_calc, building["z", ])
		result_q[pos, ] = c(t_calc, exchange["flow", ])
		result_s[pos, ] = c(t_calc, exchange["s", ])
		result_v[pos, ] = c(t_calc, exchange["flow", ] / exchange["s", ])

		# time incrementation
		t_calc = t_calc + dt_adj
	}

	# split output when invisible walls
	if (!is.null(r_names_to_merge)) {
		result_h = split_output(result_h, r_names_to_merge)
	}

	# converting water level to water height
	for(r in base_room_name) {
		result_h[, r] = result_h[, r] - base_room[["H_abs"]][base_room[["room"]] == r] 
	}
	result_h = pmax(result_h, 0)

	# find max water level by room
	building_name = c(base_room_name, colnames(bound)[-1])
	hmax = rbind(
		z = numeric(length(building_name)),
		t = numeric(length(building_name))
	)
	colnames(hmax) = building_name
	hmax["z", ] = sapply(building_name, function(r) max(result_h[, r]))
	hmax["t", ] = sapply(building_name, function(r) ifelse(hmax["z", r] != 0.0, 
		result_h[result_h[, r] == max(result_h[, r]), "time"], 0.0))
	hmax = cbind(hmax, mean = c(mean(hmax["z", ][building_name]), 
		mean(hmax["t", ][building_name])))

	# returning output
	if (light) {
		return (
			list(
				"hmax" = hmax
			)
		)
	} else {
		return(
			list(
				"h" = result_h,
				"eQ" = result_q,
				"eS" = result_s,
				"v" = result_v,
				"hmax" = hmax
			)
		)
	}
}
