#' find the damage total of the model based on the hydraulic simulation
#'
#' @param h_max_by_room vector of doubles, max water level by room
#' @param model either a character, name given to the model or an object of
#' 		class model
#' @param external character or list of characters, giving the names of the walls concerned by external limnigraphs
#'
#' @return matrix, damages for each rooms and externals based on water level
#' 
#' @examples
#' 
#' # declaring input and output paths
#' model_path = list(
#' 	data = system.file("extdata", package = "floodam.building"),
#' 	output = tempdir()
#' )
#' 
#' # analyzing model 'adu_t' of type 'adu' using the sequential steps *load* 
#' # (load model data), *extract* (extract model data) *hydraulic* 
#' # and *damaging*
#' model = analyse_model(
#' 	model = "adu_t", 
#' 	type = "adu", 
#' 	stage = c("load", "extract", "hydraulic", "damaging"), 
#' 	path = model_path,
#'  dam_by_room = TRUE
#' )
#' 
#' # generate limnigraph
#' flood = generate_limnigraph(
#'   time = c(0, 300, 900),
#'   depth = cbind(external_1 = c(0, 3, 0)),
#'   external = list(
#'     external_1 = c("wall_A", "wall_B", "wall_C", "wall_D", "wall_E", "wall_F", "wall_G", "wall_H"))
#' )
#' 
#' hydraulic = analyse_hydraulic(
#' 	model = model,
#' 	limnigraph = flood,
#' 	sim_id = "test",
#' 	stage = c("hydraulic", "damaging")
#' )
#' 
#' hmax = c(room_1 = 0.38, room_2 = 0.4, room_3 = 0.38, room_4 = 0.38, 
#'  external_1 = 3)
#' get_hydraulic_damage(hmax, model, flood[["external"]])
#'
#' @export

get_hydraulic_damage = function(h_max_by_room, model, external) {

    damg = model[["damaging"]][["absolute_by_room"]]

    # removing mean water height from calculation
    h_max_by_room = h_max_by_room[names(h_max_by_room) != "mean"]

    # convert to centimeter and round to closest decimeter
    hmax = round(h_max_by_room, 1) * 100

    damgmax = rbind(
        damages = numeric(length(hmax) + 1)
	)
    colnames(damgmax) = c(names(hmax), "total")

    for (h in names(hmax)) {
        # find wall damage for externals
        if (grepl("external", h)) {
            for (w in external[[h]]) {
                damgmax["damages", h] = damgmax["damages", h] + damg[["individual_external_wall"]][["external_wall"]][[w]][["total"]][as.character(hmax[[h]]), 2]
            }
        } else {
            # damage for rooms
            damgmax["damages", h] = damg[[h]][["total"]][as.character(hmax[[h]]), 2]
        }
    }

    damgmax["damages", "total"] = sum(damgmax["damages", ])

    return(damgmax)
}
