#' @title group elementary damages by room.
#' 
#' @param x the list of elementary components in a model. Usually the list in 
#' the slot "detail" of the slot "damaging" of a model
#' 
#' @param rooms character vector with the rooms in the dwelling
#' 
#' @param external_wall character list with the walls in the external_wall of
#'  the dwelling by storey
#' 
#' @details This function is meant to be called during the calculation of damage
#'  functions
#' 
#' @return a list 
#' 
#' @examples
#' model = analyse_model(
#'  model = "adu_t",
#'  type = "adu",
#'  stage = c("load", "extract", "damaging"),
#'  path = list(
#'    data = system.file("extdata", package = "floodam.building"),
#'    output = tempdir()
#'   )
#' )
#' test = group_damage_detail_by_room(
#'  x = model[["damaging"]][["detail"]], 
#'  rooms = levels(model[["data_table"]][["room"]][["room"]]),
#'  external_wall = as.character(
#'    model[["data_table"]][["wall"]][
#'      model[["data_table"]][["wall"]][["room"]] == "external_wall", "wall"
#'    ]
#'   )
#' )
#' 
#' @seealso [extract_elementary_component_from_detail()] and 
#' [group_damage_detail_external_wall()], to handle the calculation of 
#' damage functions by rooms
#' 
#' @export 
#' 

group_damage_detail_by_room = function(x, rooms, external_wall){
  
  # filtering potential null slots in x
  selection = !unlist(lapply(x, is.null))
  selection = names(x[selection])
  
  # initializing liqt
  result = list()

  # extracting elementary components from detail by room
  for (room in rooms){
    result[[room]] = sapply(
      selection,
      FUN = extract_elementary_component_from_detail, 
      x = x, 
      room = room, 
      simplify = FALSE
    )
  }

  if (isFALSE(any(grepl("external_wall", names(result))))) {
    warning("The 'external_wall*' slot has not be found. The damage by external wall is not calculated")
    return(result)
  }

  result[["individual_external_wall"]] = sapply(
    names(external_wall),
    function(y) {
      group_damage_detail_external_wall(result[[y]], wall = external_wall[[y]])
    },
    USE.NAMES = TRUE, 
    simplify = FALSE
  )

  #  returning result
  return(result)
}

#' @title group elementary damages by supra-elementary component
#' 
#' @description 
#'  group elementary damages by supra-elementary component (floor, ceiling, wall,
#'  opening, coating and furniture).
#' 
#' @param x the list of elementary components in a model. Usually the list in 
#' the slot "detail" of the slot "damaging" of a model
#' 
#' @param what the name of one of the categories in which the slots of the list
#' "detail" of the slot "damaging" of a model are grouped: "room" (for floor and
#' ceiling), "wall", "opening", "coating", "furniture
#' 
#' @param room the name of the room for which we want to group elementary
#' components
#' 
#' @details This function is meant to be called during the calculation of damage
#'  functions
#' 
#' @return a list of elementary components
#' 
#' @examples
#' model = analyse_model(
#'  model = "adu_t",
#'  type = "adu",
#'  stage = c("load", "extract", "damaging"),
#'  path = list(
#'    data = system.file("extdata", package = "floodam.building"),
#'    output = tempdir()
#'   )
#' )
#' test = extract_elementary_component_from_detail(
#'  what = names(model[["damaging"]][["detail"]])[2],
#'  x = model[["damaging"]][["detail"]], 
#'  room = levels(model[["data_table"]][["room"]][["room"]])[2]
#' )
#' 
#' @seealso [group_damage_detail_by_room()] and 
#' [group_damage_detail_external_wall()], to handle the calculation of 
#' damage functions by rooms
#' 
#' @export 
#' 


extract_elementary_component_from_detail = function(x, what, room){
  
  # Since the "room label" is different for "furniture" we set up a variable 
  # depending on the value of the parameter 'what' 
  if (what == "furniture"){
    label = "room_name"
  } else {
    label = "room"
  }
  
  # creating sequence with all the elements in the slot 'what' of 'x'
  id = seq_along(names(x[[what]]))
  
  # scrolling the list and retaining elements in the room indicated in 'room'
  selection = unlist(
    lapply(
      id, 
      function(id){
        attr(x[[what]][[id]], "detail")[[label]] == room
      }
    )
  )

  result = x[[what]][selection]

  #  returning result
  if (!length(result)) {
    return(NULL)
  } else {
    return(result)
  }
}

#' @title group elementary components in external_wall by wall.
#' 
#' @description 
#'  `group_damage_detail_external_wall()` is an **internal** function that
#'   groups elementary components in external_wall by wall.
#' 
#' @param x list of elementary components corresponding to the external wall of 
#'  a given storey organized and named according to floodam.building's standard
#'  
#' @param wall character, the names of the exterior walls in the storey
#' 
#' @details This function is meant to be called during the calculation of damage
#'  functions
#' 
#' @return a list of elementary components 
#' 
#' @seealso [group_damage_detail_by_room()] and 
#' [extract_elementary_component_from_detail()], to handle the calculation of 
#' damage functions by rooms
#' 
#' @keywords internal 
#' 

group_damage_detail_external_wall = function(x, wall) {

  result = list()
  for (segment in wall){
    for (what in names(x)) {

      n = length(x[[what]])
      
      selection = lapply(
        1:n, 
        function(i, what, segment) {
          attr(x[[what]][[i]], "detail")[["wall"]] == segment
        },
        what = what,
        segment = segment
      )
      
      selection = unlist(selection)
      
      if (any(selection)) {
        result[[segment]][[what]] = 
          x[[what]][selection]
      }
    }
  }
  return(result)
}
