#' Identify rooms to merge when there's invisible walls
#'
#' @param model An object of class model
#'
#' @description
#' Groups rooms that should be merged if they are separated by an invisible wall.
#' 
#' @return List of vectors of rooms to be merged

find_rooms_to_merge = function(model) {

  rooms = model$data_table$wall[c("name", "room_name")]
  rooms$room_name = as.character(rooms$room_name)
  # Remove missing values
  rooms = rooms[stats::complete.cases(rooms), ]
  # Group room names by wall name
  match = split(rooms$room_name, rooms$name)

  # Merge connected room groups
  groups = list()
  for (connex in match) {
    found = FALSE
    for(i in seq(groups)) {
      if(any(connex %in% groups[[i]])) {
        groups[[i]] = unique(c(groups[[i]], connex))
        found = TRUE
      }
    }
    if(!found) {
      groups = append(groups, list(connex))
    }
  }
  return(groups)
}

#' merge rooms
#'
#' @param opening dataframe containing opening information
#' @param room dataframe containing room information
#' @param groups List of room vectors to be merged
#'
#' @description
#' - Merges rooms separated by invisible walls.
#' - Creates new room names and sums their areas.
#' - Updates openings to match the new merged room names.
#' 
#' @return List with updated `opening` and `room` dataframes

merge_rooms = function(opening, room, groups) {
  for (g in groups) {
    new_name = paste(g, collapse = "-")

    # Update openings to reflect merged rooms
    opening[["down"]][opening[["down"]] %in% g] = new_name
    opening[["up"]][opening[["up"]] %in% g] = new_name

    # Merge areas for same level rooms and update room names
    for (level in unique(room[["H_abs"]][room[["room"]] %in% g])) {
      room[["surface"]][room[["room"]] %in% g & room[["H_abs"]] == level] = sum(room[["surface"]][room[["room"]] %in% g & room[["H_abs"]] == level])
    }
    room[["room"]][room[["room"]] %in% g] = new_name
    # Remove duplicate room entries
    room = room[!duplicated(room), ]
  }
  return(list("opening" = opening, "room" = room))
}

#' split output
#'
#' @param out_hyd data.frame of hydraulic values: water exchanges through openings and
#' 	floodwater depth in rooms
#' @param groups List of vectors of rooms to be merged
#'
#' @description
#' Based on lists of rooms with invisible walls:
#' Copy the data for merged rooms in new branches to match the model rooms.
#' 
#' @return data.frame of hydraulic values: water exchanges through openings and
#' 	floodwater depth in rooms

split_output = function(out_hyd, groups) {

  for (g in groups) {
    # find the merged room and extract it
    ind = grepl(g[[1]], colnames(out_hyd), fixed = TRUE)
    new_cols = out_hyd[, ind, drop = FALSE]

    # indhmax = grepl(g[[1]], colnames(out_hyd[["hmax"]]), fixed = TRUE) 
    # new_cols_hmax = out_hyd[["hmax"]][, indhmax, drop = FALSE] 

    # duplicate and rename the new columns
    new_cols = do.call(cbind, rep(list(new_cols), length(g)))
    colnames(new_cols) = rep(g, length.out = ncol(new_cols))

    # new_cols_hmax = do.call(cbind, rep(list(new_cols_hmax), length(g)))
    # colnames(new_cols_hmax) = rep(g, length.out = ncol(new_cols_hmax))
    
    # add them to the output
    out_hyd = cbind(out_hyd, new_cols)

    # out_hyd[["hmax"]] = cbind(out_hyd[["hmax"]], new_cols_hmax)

    # removing merged room
    new_name = paste(g, collapse = "-")
    out_hyd = out_hyd[, colnames(out_hyd) != new_name]
    # out_hyd[["hmax"]] = out_hyd[["hmax"]][, colnames(out_hyd[["hmax"]]) != new_name]

  }
  
  return(out_hyd)
}